@@ -12,14 +12,14 @@ setClass("SimSem", representation(pt = "list", dgen = "list", modelType = "chara
     groupLab = "character", con = "list"))
 
 # Set a null class to make sure that the default setting is used in data distribution object
-setClass("NullCopula", contains="copula")
+setClass("NullCopula", representation(p = "vector"), prototype(p = 0))
 	  
 setClass("SimDataDist", representation(p = "numeric", margins = "character", paramMargins = "list", 
-    keepScale = "logical", reverse = "vector", copula = "copula", skewness = "vector", kurtosis = "vector"), prototype(keepScale = TRUE, reverse = FALSE, copula = new("NullCopula"), skewness = NA, kurtosis = NA))
+    keepScale = "logical", reverse = "vector", copula = "ANY", skewness = "vector", kurtosis = "vector"), prototype(keepScale = TRUE, reverse = FALSE, copula = new("NullCopula"), skewness = NA, kurtosis = NA))
 
 setClass("SimResult", representation(modelType = "character", nRep = "numeric", coef = "data.frame", 
     se = "data.frame", fit = "data.frame", converged = "vector", paramValue = "data.frame", 
-    misspecValue = "data.frame", popFit = "data.frame", FMI1 = "data.frame", FMI2 = "data.frame", 
+    misspecValue = "data.frame", popFit = "data.frame", FMI1 = "data.frame", FMI2 = "data.frame", cilower = "data.frame", ciupper = "data.frame",
     stdCoef = "data.frame", seed = "numeric", n = "vector", nobs="data.frame", pmMCAR = "vector", pmMAR = "vector", 
     extraOut = "list", paramOnly = "logical", timing = "list"))
 
